@@ -38,12 +38,12 @@ use X11::Xlib;
 use Carp qw(carp croak);
 use XSLoader;
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 use constant DOM_TYPE_ELEMENT => 1;
 use constant ORDERED_NODE_SNAPSHOT_TYPE => 7;
 
-XSLoader::load;
+XSLoader::load(__PACKAGE__, $VERSION);
 
 has xvfb => (
     is  => 'ro',
@@ -99,6 +99,12 @@ has console_messages => (
     default => sub { [] },
 );
 
+has print_requests => (
+    is      => 'rw',
+    isa     => 'ArrayRef',
+    default => sub { [] },
+);
+
 has default_timeout => (
     is      => 'rw',
     isa     => 'Int',
@@ -120,6 +126,12 @@ has modifiers => (
     default => sub { {control => 0} },
 );
 
+=head3 init
+
+Initializes Webkit and GTK3. Must be called before any of the other methods.
+
+=cut
+
 sub init {
     my ($self) = @_;
 
@@ -147,6 +159,10 @@ sub init {
         push @{ $self->console_messages }, $_[1];
         return FALSE;
     });
+    $self->view->signal_connect('print-requested' => sub {
+        push @{ $self->print_requests }, $_[1];
+        return TRUE;
+    });
 
     $self->window->show_all;
     Gtk3->main_iteration while Gtk3->events_pending;
@@ -160,6 +176,11 @@ sub setup_xvfb {
     open my $stderr, '>&', \*STDERR or die "Can't dup STDERR: $!";
     close STDERR;
 
+    if (system('Xvfb -help') != 0) {
+        open STDERR, '>&', $stderr;
+        die 'Could not start Xvfb';
+    }
+
     my ($server, $display);
     while (1) {
         $display = 1 + int(rand(98));
@@ -181,12 +202,26 @@ sub DESTROY {
     kill 15, $self->xvfb_pid;
 }
 
+=head2 Implemented methods of the Selenium API
+
+Please see L<WWW::Selenium> for the full documentation of these methods.
+
+=head3 set_timeout($timeout)
+
+Set the default timeout to $timeout.
+
+=cut
+
 sub set_timeout {
     my ($self, $timeout) = @_;
 
     $self->default_timeout($timeout);
 }
 
+=head3 open($url)
+
+=cut
+
 sub open {
     my ($self, $url) = @_;
 
@@ -195,6 +230,10 @@ sub open {
     Gtk3->main_iteration while Gtk3->events_pending or $self->view->get_load_status ne 'finished';
 }
 
+=head3 refresh()
+
+=cut
+
 sub refresh {
     my ($self) = @_;
 
@@ -202,6 +241,10 @@ sub refresh {
     Gtk3->main_iteration while Gtk3->events_pending or $self->view->get_load_status ne 'finished';
 }
 
+=head3 go_back()
+
+=cut
+
 sub go_back {
     my ($self) = @_;
 
@@ -212,8 +255,8 @@ sub go_back {
 sub eval_js {
     my ($self, $js) = @_;
 
-    my $fn = "___run_js_$$";
-    $self->view->execute_script("function $fn() { $js }; alert($fn());");
+    $js =~ s/'/\\'/g;
+    $self->view->execute_script("alert(eval('$js'));");
     Gtk3->main_iteration while Gtk3->events_pending or $self->view->get_load_status ne 'finished';
     return pop @{ $self->alerts };
 }
@@ -246,9 +289,15 @@ sub resolve_locator {
     if (my ($label) = $locator =~ /^label=(.*)/) {
         return $self->resolve_locator($label eq '' ? qq{xpath=.//*[not(text())]} : qq{xpath=.//*[text()="$label"]}, $document, $context);
     }
+    elsif (my ($link) = $locator =~ /^link=(.*)/) {
+        return $self->resolve_locator($link eq '' ? qq{xpath=.//a[not(descendant-or-self::text())]} : qq{xpath=.//a[descendant-or-self::text()="$link"]}, $document, $context);
+    }
     elsif (my ($value) = $locator =~ /^value=(.*)/) {
         return $self->resolve_locator(qq{xpath=.//*[\@value="$value"]}, $document, $context);
     }
+    elsif (my ($index) = $locator =~ /^index=(.*)/) {
+        return $self->resolve_locator(qq{xpath=.//option[position()="$index"]}, $document, $context);
+    }
     elsif (my ($id) = $locator =~ /^id=(.*)/) {
         return $document->get_element_by_id($id);
     }
@@ -273,6 +322,10 @@ sub resolve_locator {
     die "unknown locator $locator";
 }
 
+=head3 get_xpath_count
+
+=cut
+
 sub get_xpath_count {
     my ($self, $xpath) = @_;
 
@@ -282,6 +335,10 @@ sub get_xpath_count {
     return $xpath_results->get_snapshot_length;
 }
 
+=head3 select($select, $option)
+
+=cut
+
 sub select {
     my ($self, $select, $option) = @_;
 
@@ -308,6 +365,10 @@ sub select {
     return;
 }
 
+=head3 click($locator)
+
+=cut
+
 sub click {
     my ($self, $locator) = @_;
 
@@ -321,13 +382,68 @@ sub click {
     return 1;
 }
 
+=head3 check($locator)
+
+=cut
+
+sub check {
+    my ($self, $locator) = @_;
+
+    my $element = $self->resolve_locator($locator);
+    return $self->change_check($element, 1);
+
+    return;
+}
+
+=head3 uncheck($locator)
+
+=cut
+
+sub uncheck {
+    my ($self, $locator) = @_;
+
+    my $element = $self->resolve_locator($locator);
+    return $self->change_check($element, undef);
+
+    return;
+}
+
+sub change_check {
+    my ($self, $element, $set_checked) = @_;
+
+    my $document = $self->view->get_dom_document;
+
+    unless ($set_checked) {
+        $element->remove_attribute('checked');
+    }
+    else {
+        $element->set_attribute('checked', 'checked');
+    }
+
+    my $changed = $document->create_event('Event');
+    $changed->init_event('change', TRUE, TRUE);
+    $element->dispatch_event($changed);
+
+    Gtk3->main_iteration while Gtk3->events_pending or $self->view->get_load_status ne 'finished';
+    return 1;
+}
+
+=head3 wait_for_page_to_load($timeout)
+
+=cut
+
 sub wait_for_page_to_load {
     my ($self, $timeout) = @_;
 
+    #TODO implement timeout
     $self->pause(300);
     Gtk3->main_iteration while Gtk3->events_pending or $self->view->get_load_status ne 'finished';
 }
 
+=head3 wait_for_element_present($locator, $timeout)
+
+=cut
+
 sub wait_for_element_present {
     my ($self, $locator, $timeout) = @_;
     $timeout ||= $self->default_timeout;
@@ -346,22 +462,9 @@ sub wait_for_element_present {
     return $element;
 }
 
-sub wait_for_element_to_disappear {
-    my ($self, $locator, $timeout) = @_;
-    $timeout ||= $self->default_timeout;
+=head3 is_element_present($locator)
 
-    my $element;
-    my $expiry = time + $timeout / 1000;
-
-    while ($element = $self->is_element_present($locator)) {
-        Gtk3->main_iteration while Gtk3->events_pending;
-
-        return 0 if time > $expiry;
-        usleep 10000;
-    }
-
-    return 1;
-}
+=cut
 
 sub is_element_present {
     my ($self, $locator) = @_;
@@ -369,6 +472,10 @@ sub is_element_present {
     return eval { $self->resolve_locator($locator) };
 }
 
+=head3 get_text($locator)
+
+=cut
+
 sub get_text {
     my ($self, $locator) = @_;
 
@@ -379,6 +486,10 @@ sub get_text {
     return $value;
 }
 
+=head3 type($locator, $text)
+
+=cut
+
 sub type {
     my ($self, $locator, $text) = @_;
 
@@ -413,6 +524,10 @@ sub key_press {
     return 1;
 }
 
+=head3 type_keys($locator, $string)
+
+=cut
+
 sub type_keys {
     my ($self, $locator, $string) = @_;
 
@@ -437,6 +552,10 @@ sub control_key_up {
     $self->modifiers->{control} = 0;
 }
 
+=head3 pause($time)
+
+=cut
+
 sub pause {
     my ($self, $time) = @_;
 
@@ -454,23 +573,39 @@ sub pause {
     }
 }
 
+=head3 is_ordered($first, $second)
+
+=cut
+
 sub is_ordered {
     my ($self, $first, $second) = @_;
-    return 1;
+    return $self->resolve_locator($first)->compare_document_position($self->resolve_locator($second)) == 4;
 }
 
+=head3 get_body_text()
+
+=cut
+
 sub get_body_text {
     my ($self) = @_;
 
     return $self->get_text('xpath=//body');
 }
 
+=head3 get_title()
+
+=cut
+
 sub get_title {
     my ($self) = @_;
 
     return $self->get_text('xpath=//title');
 }
 
+=head3 mouse_over($locator)
+
+=cut
+
 sub mouse_over {
     my ($self, $locator) = @_;
 
@@ -484,6 +619,10 @@ sub mouse_over {
     return 1;
 }
 
+=head3 mouse_down($locator)
+
+=cut
+
 sub mouse_down {
     my ($self, $locator) = @_;
 
@@ -496,6 +635,10 @@ sub mouse_down {
     return 1;
 }
 
+=head3 fire_event($locator, $event_type)
+
+=cut
+
 sub fire_event {
     my ($self, $locator, $event_type) = @_;
 
@@ -509,6 +652,10 @@ sub fire_event {
     return 1;
 }
 
+=head3 get_value($locator)
+
+=cut
+
 sub get_value {
     my ($self, $locator) = @_;
 
@@ -524,6 +671,10 @@ sub get_value {
     }
 }
 
+=head3 get_attribute($locator)
+
+=cut
+
 sub get_attribute {
     my ($self, $locator) = @_;
     ($locator, my $attr) = $locator =~ m!\A (.*?) /?@ ([^@]*) \z!xm;
@@ -531,6 +682,10 @@ sub get_attribute {
     return $self->resolve_locator($locator)->get_attribute($attr);
 }
 
+=head3 is_visible($locator)
+
+=cut
+
 sub is_visible {
     my ($self, $locator) = @_;
 
@@ -550,6 +705,10 @@ sub is_visible {
     return $visible;
 }
 
+=head3 submit($locator)
+
+=cut
+
 sub submit {
     my ($self, $locator) = @_;
 
@@ -559,6 +718,10 @@ sub submit {
     return 1;
 }
 
+=head3 get_html_source()
+
+=cut
+
 sub get_html_source {
     my ($self) = @_;
 
@@ -567,12 +730,30 @@ sub get_html_source {
     return $data;
 }
 
+=head3 get_confirmation()
+
+=cut
+
 sub get_confirmation {
     my ($self) = @_;
 
     return pop @{ $self->confirmations };
 }
 
+=head3 print_requested()
+
+=cut
+
+sub print_requested {
+    my ($self) = @_;
+
+    return pop @{ $self->print_requests } ? 1 : 0;
+}
+
+=head3 answer_on_next_prompt($answer)
+
+=cut
+
 sub answer_on_next_prompt {
     my ($self, $answer) = @_;
 
@@ -581,6 +762,56 @@ sub answer_on_next_prompt {
 
 =head2 Additions to the Selenium API
 
+=head3 wait_for_element_to_disappear($locator, $timeout)
+
+Works just like wait_for_element_present but instead of waiting for the element to appear, it waits for the element to disappear.
+
+=cut
+
+sub wait_for_element_to_disappear {
+    my ($self, $locator, $timeout) = @_;
+    $timeout ||= $self->default_timeout;
+
+    my $element;
+    my $expiry = time + $timeout / 1000;
+
+    while ($element = $self->is_element_present($locator)) {
+        Gtk3->main_iteration while Gtk3->events_pending;
+
+        return 0 if time > $expiry;
+        usleep 10000;
+    }
+
+    return 1;
+}
+
+=head3 wait_for_alert($text, $timeout)
+
+Wait for an alert with the given text to happen.
+If $text is undef, it waits for any alert. Since alerts do not get automatically cleared, this has to be done manually before causing the action that is supposed to throw a new alert:
+
+    $webkit->alerts([]);
+    $webkit->click('...');
+    $webkit->wait_for_alert;
+
+=cut
+
+sub wait_for_alert {
+    my ($self, $text, $timeout) = @_;
+    $timeout ||= $self->default_timeout;
+
+    my $expiry = time + $timeout / 1000;
+
+    until (defined $text ? (@{ $self->alerts } and $self->alerts->[-1] eq $text) : @{ $self->alerts }) {
+        Gtk3->main_iteration while Gtk3->events_pending;
+
+        return 0 if time > $expiry;
+        usleep 10000;
+    }
+
+    return 1;
+}
+
 =head3 native_drag_and_drop_to_object($source, $target)
 
 Drag&drop that works with native HTML5 D&D events.
@@ -653,8 +884,8 @@ sub get_center_screen_position {
 =head1 SEE ALSO
 
 See L<WWW::Selenium> for API documentation.
-L<Test::WWW::WebKit> for a replacement for L<Test::WWW::Selenium>
-L<Test::WWW::WebKit::Catalyst> for a replacement for L<Test::WWW::Selenium::Catalyst>
+See L<Test::WWW::WebKit> for a replacement for L<Test::WWW::Selenium>.
+See L<Test::WWW::WebKit::Catalyst> for a replacement for L<Test::WWW::Selenium::Catalyst>.
 
 =head1 AUTHOR
 
